var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var AppBabylonFps = (function (_super) {
    __extends(AppBabylonFps, _super);
    function AppBabylonFps() {
        var _this = this;
        _super.apply(this, arguments);
        this.cubes = [];
        this.controls = new Controls(function () { return _this.changeCubesCount(false); }, function () { return _this.changeCubesCount(true); });
        this.testCounts = [1, 50, 500, 1000, 2000, 5000];
    }
    AppBabylonFps.prototype.main = function () {
        this.createScene();
        this.registerResizeHandler();
        this.addStats();
        this.addControls();
        this.addCamera();
        this.addLights();
        this.createCube();
        this.addCubes(this.controls.cubesCount);
        this.renderWithAnimation();
    };
    AppBabylonFps.prototype.renderWithAnimation = function () {
        var _this = this;
        this.engine.runRenderLoop(function () {
            _this.scene.render();
            if (_this.controls.animation) {
                for (var i = 0; i < _this.cubes.length; i++) {
                    switch (i % 3) {
                        case 0:
                            _this.cubes[i].rotation.x += Math.PI / 180;
                            break;
                        case 1:
                            _this.cubes[i].rotation.y += Math.PI / 180;
                            break;
                        case 2:
                            _this.cubes[i].rotation.z += Math.PI / 180;
                            break;
                    }
                }
            }
            _this.stats.update();
            _this.statsMs.update();
        });
    };
    AppBabylonFps.prototype.addCubes = function (cubesCount) {
        this.addStats();
        while (this.cubes.length > 1) {
            var cubeToDelete = this.cubes.pop();
            this.scene.getMeshByName(cubeToDelete.name).dispose();
        }
        for (var i = 2; i <= cubesCount; i++) {
            var newCube = this.cube.createInstance("cube" + i.toString());
            this.cubes.push(newCube);
        }
        var addedCubesCount = 0;
        var squareSize = Math.ceil(Math.sqrt(cubesCount));
        var xdelta = 20 / squareSize;
        var ydelta = 15 / squareSize;
        var startk = Math.floor(-squareSize / 2);
        for (var k = startk; k < startk + squareSize; k++) {
            var startl = Math.floor(-squareSize / 2);
            for (var l = startl; l < startl + squareSize; l++) {
                if (addedCubesCount < cubesCount) {
                    var cube = this.cubes[addedCubesCount];
                    cube.position.x = (l + 1) * xdelta;
                    cube.position.y = (k + 1) * ydelta;
                    cube.position.z = 0;
                    cube.rotation.x = Math.random();
                    cube.rotation.y = Math.random();
                    cube.rotation.z = Math.random();
                    addedCubesCount++;
                }
                else {
                    break;
                }
            }
        }
    };
    AppBabylonFps.prototype.addLights = function () {
        var light1 = new BABYLON.DirectionalLight("light1", new BABYLON.Vector3(-0.5, -0.5, 1), this.scene);
        light1.specular = new BABYLON.Color3(0, 0, 0);
    };
    AppBabylonFps.prototype.addControls = function () {
        var _this = this;
        var gui = new dat.GUI();
        var controller = gui.add(this.controls, 'cubesCount', 1, 5000).step(1);
        controller.setName('Počet krychlí');
        var controllerPre = gui.add(this.controls, 'cubesCount', [1, 50, 500, 1000, 2000, 5000]);
        controllerPre.setName('Počet krychlí');
        var controllerAnimation = gui.add(this.controls, 'animation');
        controllerAnimation.setName('Animace');
        var controllerTest = gui.add(this.controls, 'test');
        controllerTest.setName('Test');
        var controllerTestAnimated = gui.add(this.controls, 'testAnimated');
        controllerTestAnimated.setName('Test - animace');
        controller.onFinishChange(function (value) { return _this.addCubes(value); });
        controllerPre.onFinishChange(function (value) { return _this.addCubes(value); });
    };
    AppBabylonFps.prototype.addCamera = function () {
        this.camera = new BABYLON.ArcRotateCamera("camera", 3 * Math.PI / 2, Math.PI / 2, 20, new BABYLON.Vector3(0, 0, 0), this.scene);
        this.camera.position = new BABYLON.Vector3(0, 0, 20);
        this.scene.activeCamera.attachControl(this.canvas);
    };
    AppBabylonFps.prototype.createCube = function () {
        this.cube = BABYLON.Mesh.CreateBox("cube1", 1.0, this.scene);
        this.cubes.push(this.cube);
        this.material = new BABYLON.StandardMaterial("material1", this.scene);
        this.material.diffuseColor = new BABYLON.Color3(0.35, 0.76, 0.8);
        this.cube.material = this.material;
    };
    AppBabylonFps.prototype.changeCubesCount = function (animated) {
        var self = this;
        if (!animated) {
            self.controls.animation = false;
        }
        else {
            self.controls.animation = true;
        }
        for (var i = 0; i <= self.testCounts.length; i++)
            setTimeout(function (j) {
                if (j > 0) {
                    var medianFps = self.stats.getFpsMedian();
                    var medianMs = self.stats.getMsMedian();
                    console.log(self.testCounts[j - 1] + ": " + medianFps.toString() + ", " + medianMs.toString());
                    if (j === self.testCounts.length) {
                        alert("Test dokončen.");
                        self.addCubes(self.testCounts[0]);
                        self.controls.animation = false;
                    }
                }
                if (j < self.testCounts.length) {
                    self.addCubes(self.testCounts[j]);
                }
            }, 10000 * i, i);
    };
    AppBabylonFps.prototype.writeFps = function (count) {
        var self = this;
        setTimeout(function (j) {
            var medianFps = self.stats.getFpsMedian();
            var medianMs = self.stats.getMsMedian();
            console.log(j.toString() + ": " + medianFps.toString() + ", " + medianMs.toString());
        }, 17000, count);
    };
    return AppBabylonFps;
})(AppBabylonBase);
window.onload = function () {
    var app = new AppBabylonFps();
    app.main();
};
