var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var AppThreeFps = (function (_super) {
    __extends(AppThreeFps, _super);
    function AppThreeFps() {
        var _this = this;
        _super.call(this);
        this.cubes = [];
        this.controls = new Controls(function () { return _this.changeCubesCount(false); }, function () { return _this.changeCubesCount(true); });
        this.testCounts = [1, 50, 500, 1000, 2000, 5000];
    }
    AppThreeFps.prototype.main = function () {
        this.addCamera();
        this.createScene();
        this.addStats();
        this.addControls();
        this.addLights();
        this.defineCube();
        this.addCubes(this.controls.cubesCount);
        this.registerResizeHandler();
        this.renderWithAnimation();
    };
    AppThreeFps.prototype.addCubes = function (cubesCount) {
        this.addStats();
        while (this.cubes.length > 0) {
            var cube = this.cubes.pop();
            this.scene.remove(cube);
        }
        var addedCubesCount = 0;
        var squareSize = Math.ceil(Math.sqrt(cubesCount));
        var xdelta = 20 / squareSize;
        var ydelta = 15 / squareSize;
        var startk = Math.floor(-squareSize / 2);
        for (var k = startk; k < startk + squareSize; k++) {
            var startl = Math.floor(-squareSize / 2);
            for (var l = startl; l < startl + squareSize; l++) {
                var newCube = new THREE.Mesh(this.geometry, this.material);
                if (addedCubesCount < cubesCount) {
                    newCube.position.x = (l + 1) * xdelta;
                    newCube.position.y = (k + 1) * ydelta;
                    newCube.position.z = 0;
                    newCube.rotation.x = Math.random();
                    newCube.rotation.y = Math.random();
                    newCube.rotation.z = Math.random();
                    addedCubesCount++;
                    this.cubes.push(newCube);
                    this.scene.add(newCube);
                }
                else {
                    break;
                }
            }
        }
    };
    AppThreeFps.prototype.renderWithAnimation = function () {
        var _this = this;
        requestAnimationFrame(function () { return _this.renderWithAnimation(); });
        if (this.controls.animation) {
            for (var i = 0; i < this.cubes.length; i++) {
                switch (i % 3) {
                    case 0:
                        this.cubes[i].rotation.x += Math.PI / 180;
                        break;
                    case 1:
                        this.cubes[i].rotation.y += Math.PI / 180;
                        break;
                    case 2:
                        this.cubes[i].rotation.z += Math.PI / 180;
                        break;
                }
            }
        }
        this.stats.update();
        this.statsMs.update();
        this.renderer.render(this.scene, this.camera);
    };
    AppThreeFps.prototype.addControls = function () {
        var _this = this;
        var gui = new dat.GUI();
        var controller = gui.add(this.controls, 'cubesCount', 1, 5000).step(1);
        controller.setName('Počet krychlí');
        var controllerPre = gui.add(this.controls, 'cubesCount', [1, 50, 500, 1000, 2000, 5000]);
        controllerPre.setName('Počet krychlí');
        var controllerAnimation = gui.add(this.controls, 'animation');
        controllerAnimation.setName('Animace');
        var controllerTest = gui.add(this.controls, 'test');
        controllerTest.setName('Test');
        var controllerTestAnimated = gui.add(this.controls, 'testAnimated');
        controllerTestAnimated.setName('Test - animace');
        controller.onFinishChange(function (value) { return _this.addCubes(value); });
        controllerPre.onFinishChange(function (value) { return _this.addCubes(value); });
    };
    AppThreeFps.prototype.addCamera = function () {
        this.camera = new THREE.PerspectiveCamera(45, window.innerWidth / window.innerHeight, 0.01, 200);
        this.camera.position.set(0, 0, 20);
    };
    AppThreeFps.prototype.defineCube = function () {
        this.material = new THREE.MeshLambertMaterial({ color: 0x5bc2ce });
        this.geometry = new THREE.BoxGeometry(1, 1, 1, 1, 1, 1);
    };
    AppThreeFps.prototype.addLights = function () {
        var light = new THREE.DirectionalLight(0xffffff, 1);
        light.position.set(0.5, 0.5, 1);
        this.scene.add(light);
    };
    AppThreeFps.prototype.changeCubesCount = function (animated) {
        var self = this;
        if (!animated) {
            self.controls.animation = false;
        }
        else {
            self.controls.animation = true;
        }
        for (var i = 0; i <= self.testCounts.length; i++)
            setTimeout(function (j) {
                if (j > 0) {
                    var medianFps = self.stats.getFpsMedian();
                    var medianMs = self.stats.getMsMedian();
                    console.log(self.testCounts[j - 1] + ": " + medianFps.toString() + ", " + medianMs.toString());
                    if (j === self.testCounts.length) {
                        alert("Test dokončen.");
                        self.addCubes(self.testCounts[0]);
                        self.controls.animation = false;
                    }
                }
                if (j < self.testCounts.length) {
                    self.addCubes(self.testCounts[j]);
                }
            }, 10000 * i, i);
    };
    AppThreeFps.prototype.writeFps = function (count) {
        var self = this;
        setTimeout(function (j) {
            var medianFps = self.stats.getFpsMedian();
            var medianMs = self.stats.getMsMedian();
            console.log(j.toString() + ": " + medianFps.toString() + ", " + medianMs.toString());
        }, 17000, count);
    };
    return AppThreeFps;
})(AppThreeBase);
window.onload = function () {
    var app = new AppThreeFps();
    app.main();
};
