﻿/// <reference path="../decl/cannon.d.ts" />
/// <reference path="../decl/babylon.2.0.d.ts" />
/// <reference path="../decl/dat-gui.d.ts" />
/// <reference path="../decl/stats.d.ts" />

class AppBabylonBase {

    public engine: BABYLON.Engine;
    public canvas: HTMLCanvasElement;
    public scene: BABYLON.Scene;
    public camera: BABYLON.Camera;
    public stats: Stats;
    public statsMs: Stats;

    /*
    Postara se o opakovane vykresleni a aktualizuje statistiky. 
    */
    render() {
        this.engine.runRenderLoop(() => {
            this.scene.render();
            this.stats.update();
            this.statsMs.update();
        });
    }

    /*
    Zaregistruje obsluhu pri zmene velikosti okna. 
    */
    registerResizeHandler(): void {
        window.addEventListener("resize",() => {
            this.engine.resize();
        });
    }

    /*
    Prida statistiky. 
    */
    addStats() {
        // ms
        this.statsMs = new Stats();
        this.statsMs.setMode(1);
        document.body.appendChild(this.statsMs.domElement);

        this.statsMs.domElement.style.position = "absolute";
        this.statsMs.domElement.style.left = "10px";
        this.statsMs.domElement.style.top = "80px";

        // fps
        this.stats = new Stats();
        document.body.appendChild(this.stats.domElement);

        this.stats.domElement.style.position = "absolute";
        this.stats.domElement.style.left = "10px";
        this.stats.domElement.style.top = "10px";
    }

    /*
    Nastavi canvas a vytvori scenu.
    */
    createScene() {
        this.canvas = <HTMLCanvasElement>document.getElementById("canvas");
        this.canvas.width = window.innerWidth;
        this.canvas.height = window.innerHeight;

        this.engine = new BABYLON.Engine(this.canvas, true);

        this.scene = new BABYLON.Scene(this.engine);
        this.scene.clearColor = BABYLON.Color3.Black();
    }
}