﻿/// <reference path="../decl/babylon.2.0.d.ts" />
/// <reference path="../decl/cannon.d.ts" />

class AppBabylonGeometry extends AppBabylonBase {

    public main() {
        this.createScene();
        this.registerResizeHandler();

        this.addStats();
        this.addCamera();
        this.addLights();
        this.addObjects();

        this.render();
    }

    /*
    Prida kameru. 
    */
    addCamera() {
        this.camera = new BABYLON.ArcRotateCamera
            ("camera", 3 * Math.PI / 2, Math.PI / 2, 10, new BABYLON.Vector3(0, 0, 0), this.scene);
        this.camera.position = new BABYLON.Vector3(0, 0, 1);
        this.scene.activeCamera.attachControl(this.canvas);
    }

    /*
    Prida svetla do sceny. 
    */
    addLights() {
        var light1 = new BABYLON.DirectionalLight("light1", new BABYLON.Vector3(-0.5, -0.5, 1), this.scene);
        light1.diffuse = new BABYLON.Color3(1, 1, 1);
        light1.specular = new BABYLON.Color3(1, 1, 1);
    }

    /*
    Prida objekty do sceny. 
    */
    addObjects() {
        var box = BABYLON.Mesh.CreateBox("box", 1.0, this.scene);
        var materialBox = new BABYLON.StandardMaterial("material1", this.scene);
        materialBox.diffuseColor = BABYLON.Color3.Red();
        box.material = materialBox;

        box.setAbsolutePosition(new BABYLON.Vector3(-1, 0, 0));

        var sphere = BABYLON.Mesh.CreateSphere("sphere", 30, 1, this.scene);
        var materialSphere = new BABYLON.StandardMaterial("material2", this.scene);
        materialSphere.diffuseColor = BABYLON.Color3.Yellow();
        sphere.material = materialSphere;

        sphere.setAbsolutePosition(new BABYLON.Vector3(-3, 0, 0));

        var cylinder = BABYLON.Mesh.CreateCylinder("cylinder", 1, 0.5, 0.5, 30, 30, this.scene);
        var materialCylinder = new BABYLON.StandardMaterial("material3", this.scene);
        materialCylinder.diffuseColor = BABYLON.Color3.Green();
        cylinder.material = materialCylinder;

        cylinder.setAbsolutePosition(new BABYLON.Vector3(1, 0, 0));

        var torus = BABYLON.Mesh.CreateTorus("torus", 0.8, 0.4, 30, this.scene);
        var materialTorus = new BABYLON.StandardMaterial("material4", this.scene);
        materialTorus.diffuseColor = BABYLON.Color3.Blue();
        torus.material = materialTorus;

        torus.setAbsolutePosition(new BABYLON.Vector3(3, 0, 0));
        torus.rotate(new BABYLON.Vector3(0, 0, 1), 1.5, BABYLON.Space.LOCAL);
        torus.rotate(new BABYLON.Vector3(1, 0, 0), 1.5, BABYLON.Space.LOCAL);
    }
}

window.onload = () => {
    var app = new AppBabylonGeometry();
    app.main();
}; 