﻿/// <reference path="../decl/babylon.2.0.d.ts" />
/// <reference path="../decl/cannon.d.ts" />

class AppBabylonLights extends AppBabylonBase {

    public main() {
        this.createScene();
        this.registerResizeHandler();

        this.addStats();
        this.addCamera();
        this.addLights();
        this.addObjects();

        this.render();
    }

    /*
    Prida kameru. 
    */
    addCamera() {
        this.camera = new BABYLON.ArcRotateCamera
            ("camera", 3 * Math.PI / 2, Math.PI / 2, 10, new BABYLON.Vector3(0, 0, 0), this.scene);
        this.camera.position = new BABYLON.Vector3(0, 0, 1);
        this.scene.activeCamera.attachControl(this.canvas);
    }

    /*
    Prida svetla do sceny. 
    */
    addLights() {
        var dirLight = new BABYLON.DirectionalLight("dirLight", new BABYLON.Vector3(1.5, 0, 0), this.scene);
        dirLight.diffuse = new BABYLON.Color3(0.016, 0.506, 0.62);
        dirLight.specular = new BABYLON.Color3(0.4, 0.4, 0.4);

        var pointLight = new BABYLON.PointLight("pointLight",
            new BABYLON.Vector3(1.5, 0, 0), this.scene);
        pointLight.diffuse = new BABYLON.Color3(1, 0.498, 0);
        pointLight.specular = new BABYLON.Color3(0.5, 0.5, 0.5);
    }

    /*
    Prida objekty do sceny. 
    */
    addObjects() {
        var sphere = BABYLON.Mesh.CreateSphere("sphere", 40, 1.4, this.scene);
        var materialSphere = new BABYLON.StandardMaterial("material2", this.scene);
        materialSphere.diffuseColor = BABYLON.Color3.White();
        sphere.material = materialSphere;

        var plane = BABYLON.Mesh.CreatePlane("planeBack", 4, this.scene);
        plane.position.z = 2;
        plane.scaling.x = 1.5;

        var planeDown = BABYLON.Mesh.CreatePlane("planeDown", 4, this.scene);
        planeDown.position.y = -2;
        planeDown.scaling.x = 1.5;
        planeDown.rotation.x = Math.PI / 2;

        var planeLeft = BABYLON.Mesh.CreatePlane("planeLeft", 4, this.scene);
        planeLeft.position.x = -3;
        planeLeft.rotation.y = 3 * Math.PI / 2;

        var planeRight = BABYLON.Mesh.CreatePlane("planeRight", 4, this.scene);
        planeRight.position.x = 3;
        planeRight.rotation.y = Math.PI / 2;
    }
}

window.onload = () => {
    var app = new AppBabylonLights();
    app.main();
}; 