﻿/// <reference path="../decl/cannon.d.ts" />
/// <reference path="../shared/controls.ts" />
/// <reference path="../decl/dat-gui.d.ts" />
declare var SceneJS;

class AppSceneFps extends AppSceneBase {
    private cubes: any[] = [];
    private rotateX: any;
    private rotateY: any;
    private rotateZ: any;
    private controls: Controls = new Controls(() => this.changeCubesCount(false),() => this.changeCubesCount(true));
    private testCounts: number[] = [1, 50, 500, 1000, 2000, 5000];

    public main() {
        var gui = document.createElement("div");
        gui.id = "gui-container";
        document.body.appendChild(gui);

        this.createScene();
        this.addStats();

        this.addControls();

        this.addCubes(this.controls.cubesCount);
        this.renderWithAnimation();
    }

    /*
    Postara se o vykresleni s animaci a aktualizuje statistiky. 
    */
    renderWithAnimation() {
        requestAnimationFrame(() => this.renderWithAnimation());
        if (this.controls.animation) {
            for (var i = 0; i < this.cubes.length; i++) {
                switch (i % 3) {
                    case 0:
                        this.scene.getNode("rotateX" + (i).toString(), rotateX => { rotateX.setAngle(rotateX.getAngle() + 1.0) });
                        break;
                    case 1:
                        this.scene.getNode("rotateY" + (i).toString(), rotateY => { rotateY.setAngle(rotateY.getAngle() + 1.0) });
                        break;
                    case 2:
                        this.scene.getNode("rotateZ" + (i).toString(), rotateZ => { rotateZ.setAngle(rotateZ.getAngle() + 1.0) });
                        break;
                }
            }
        }
        this.stats.update();
        this.statsMs.update();
    }

    /*
    Vytvori scenu (kameru, renderer, svetla, material)
    */
    createScene() {
        this.scene = SceneJS.createScene({
            nodes: [
                {
                    type: "cameras/orbit",
                    eye: {
                        x: 0,
                        y: 0,
                        z: 1
                    },
                    zoom: 40,
                    look: { x: 0, y: 0, z: 0 },
                    nodes: [
                        {
                            type: "renderer",
                            clearColor: { r: 0, g: 0, b: 0 },
                            clear: {
                                depth: true,
                                color: true
                            },
                            nodes: [
                                {
                                    type: "lights",
                                    lights: [
                                        {
                                            mode: "dir",
                                            color: { r: 1.0, g: 1.0, b: 1.0 },
                                            diffuse: true,
                                            specular: false,
                                            dir: { x: -0.5, y: -0.5, z: -1.0 },
                                            space: "world"
                                        }
                                    ],

                                    nodes: [
                                        {
                                            type: "material",
                                            color: { r: 1, g: 0, b: 0 },
                                            nodes: [
                                                {
                                                    type: "node",
                                                    id: "cubes"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        });
    }

    /*
    Prida ovladani GUI. 
    */
    addControls(): void {
        var gui = new dat.GUI();

        var controller = gui.add(this.controls, 'cubesCount', 1, 5000).step(1);
        controller.setName('Počet krychlí');

        var controllerPre = gui.add(this.controls, 'cubesCount', [1, 50, 500, 1000, 2000, 5000]);
        controllerPre.setName('Počet krychlí');

        var controllerAnimation = gui.add(this.controls, 'animation');
        controllerAnimation.setName('Animace');

        var controllerTest = gui.add(this.controls, 'test');
        controllerTest.setName('Test');
        var controllerTestAnimated = gui.add(this.controls, 'testAnimated');
        controllerTestAnimated.setName('Test - animace');

        controller.onFinishChange((value) => this.addCubes(value));
        controllerPre.onFinishChange((value) => this.addCubes(value));
    }

    /*
    Prida krychle do sceny. 
    */
    addCubes(cubesCount): void {
        this.addStats();
        while (this.cubes.length > 0) {
            var cube = this.cubes.pop();
            cube.name.destroy();
        }
        var addedCubesCount = 0;
        var squareSize = Math.ceil(Math.sqrt(cubesCount));
        var xdelta = 40 / squareSize;
        var ydelta = 30 / squareSize;
        var startk = Math.floor(-squareSize / 2);
        for (var k = startk; k < startk + squareSize; k++) {
            var startl = Math.floor(-squareSize / 2);
            for (var l = startl; l < startl + squareSize; l++) {
                if (addedCubesCount < cubesCount) {
                    var innerL = (l + 1) * xdelta;
                    var innerK = (k + 1) * ydelta;
                    var randomX = Math.random() * 360;
                    var randomY = Math.random() * 360;
                    var randomZ = Math.random() * 360;
                    // ReSharper disable ClosureOnModifiedVariable
                    this.scene.getNode("cubes",
                        cubeNode => {
                            var cube = {
                                name: cubeNode.addNode({
                                    type: "name",
                                    name: addedCubesCount.toString(),
                                    nodes: [
                                        {
                                            type: "material",
                                            color: { r: 0.35, g: 0.76, b: 0.8 },
                                            nodes: [
                                                {
                                                    type: "translate",

                                                    x: innerL,
                                                    y: innerK,
                                                    z: 0,
                                                    nodes: [
                                                        {
                                                            type: "rotate",
                                                            id: "rotateX" + addedCubesCount.toString(),
                                                            angle: randomX,
                                                            x: 1,
                                                            nodes: [
                                                                {
                                                                    type: "rotate",
                                                                    id: "rotateY" + addedCubesCount.toString(),
                                                                    angle: randomY,
                                                                    y: 1,
                                                                    nodes: [
                                                                        {
                                                                            type: "rotate",
                                                                            id: "rotateZ" + addedCubesCount.toString(),
                                                                            angle: randomZ,
                                                                            z: 1,
                                                                            nodes: [
                                                                                {
                                                                                    type: "geometry/box",
                                                                                    xSize: 1,
                                                                                    ySize: 1,
                                                                                    zSize: 1
                                                                                }
                                                                            ]
                                                                        }
                                                                    ]
                                                                }
                                                            ]
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                })
                            }
                            
                            // ReSharper restore ClosureOnModifiedVariable
                            addedCubesCount++;
                            this.cubes.push(cube);
                        });
                } else {
                    break;
                }
            }
        }
    }

    /*
    Zmena poctu krychli pri spusteni testu. 
    */
    changeCubesCount(animated): void {
        var self = this;
        if (!animated) {
            self.controls.animation = false;
        } else {
            self.controls.animation = true;
        }

        for (var i = 0; i <= self.testCounts.length; i++)
            setTimeout((j) => {
                if (j > 0) {
                    var medianFps = self.stats.getFpsMedian();
                    var medianMs = self.stats.getMsMedian();
                    console.log(self.testCounts[j - 1] + ": " + medianFps.toString() + ", " + medianMs.toString());
                    if (j === self.testCounts.length) {
                        alert("Test dokončen.");
                        self.addCubes(self.testCounts[0]);
                        self.controls.animation = false;
                    }
                }
                if (j < self.testCounts.length) {
                    self.addCubes(self.testCounts[j]);
                }
            }, 10000 * i, i);
    }

    /*
    Vypsani FPS a MS dat do konzole. 
    */
    writeFps(count): void {
        var self = this;
        setTimeout((j) => {
            var medianFps = self.stats.getFpsMedian();
            var medianMs = self.stats.getMsMedian();
            console.log(j.toString() + ": " + medianFps.toString() + ", " + medianMs.toString());
        }, 17000, count);
    }
}

window.onload = () => {
    var app = new AppSceneFps();
    app.main();
};