﻿/// <reference path="../decl/three.d.ts" />
/// <reference path="../decl/three-orbitcontrols.d.ts" />
/// <reference path="../decl/dat-gui.d.ts" />
/// <reference path="../decl/stats.d.ts" />
/// <reference path="../decl/threex-rendererstats.d.ts" />

class AppThreeBase {

    public scene: THREE.Scene;
    public camera: THREE.Camera;
    public renderer: THREE.WebGLRenderer;
    public stats: Stats;
    public statsMs: Stats;

    /*
    Zaregistruje obsluhu pri zmene velikosti okna. 
    */
    registerResizeHandler(): void {
        window.addEventListener('resize',() => {
            if (!this.renderer || !this.camera || !this.camera['aspect'] || !this.camera['updateProjectionMatrix'])
                return;
            this.renderer.setSize(window.innerWidth, window.innerHeight);
            (<any>this.camera).aspect = window.innerWidth / window.innerHeight;
            (<any>this.camera).updateProjectionMatrix();
        }, false);
    }

	/*
    Postara se o opakovane vykresleni a aktualizuje statistiky. 
    */
    render() {
        requestAnimationFrame(() => this.render());
        this.stats.update();
        this.statsMs.update();
        this.renderer.render(this.scene, this.camera);
    }

     /*
    Prida statistiky. 
    */
    addStats() {
        // ms
        this.statsMs = new Stats();
        this.statsMs.setMode(1);
        document.body.appendChild(this.statsMs.domElement);

        this.statsMs.domElement.style.position = "absolute";
        this.statsMs.domElement.style.left = "10px";
        this.statsMs.domElement.style.top = "80px";

        // fps
        this.stats = new Stats();
        document.body.appendChild(this.stats.domElement);

        this.stats.domElement.style.position = "absolute";
        this.stats.domElement.style.left = "10px";
        this.stats.domElement.style.top = "10px";
    }

    /*
    Vytvori scenu a renderer. 
    */
    createScene() {
        // scena
        this.scene = new THREE.Scene();

        // renderer
        this.renderer = new THREE.WebGLRenderer({ clearColor: 0x000000, alpha: true });
        this.renderer.setClearColor(new THREE.Color(0x000000));
        // pozadi
        this.renderer.setSize(window.innerWidth, window.innerHeight);

        document.body.appendChild(this.renderer.domElement);

        // ovladani sceny
        var orbitControls = new THREE.OrbitControls(this.camera, this.renderer.domElement);
    }
}