﻿/// <reference path="../decl/three.d.ts" />
/// <reference path="../decl/three-orbitcontrols.d.ts" />
/// <reference path="../decl/dat-gui.d.ts" />
/// <reference path="../decl/stats.d.ts" />
/// <reference path="../shared/controls.ts" />
/// <reference path="appthreebase.ts" />
/// <reference path="../decl/jquery.d.ts" />

class AppThreeFps extends AppThreeBase {
    private cubes: THREE.Mesh[] = [];
    private material: THREE.MeshLambertMaterial;
    private geometry: THREE.BoxGeometry;
    private controls: Controls = new Controls(() => this.changeCubesCount(false),() => this.changeCubesCount(true));
    private testCounts: number[] = [1, 50, 500, 1000, 2000, 5000];

    constructor() {
        super();
    }

    public main() {

        // kamera
        this.addCamera();
        this.createScene();

        this.addStats();
        this.addControls();

        this.addLights();
        this.defineCube();

        this.addCubes(this.controls.cubesCount);

        this.registerResizeHandler();
        this.renderWithAnimation();
    }

    /*
    Prida krychle do sceny. 
    */
    addCubes(cubesCount): void {
        this.addStats();
        while (this.cubes.length > 0) {
            var cube = this.cubes.pop();
            this.scene.remove(cube);
        }
        var addedCubesCount = 0;
        var squareSize = Math.ceil(Math.sqrt(cubesCount));
        var xdelta = 20 / squareSize;
        var ydelta = 15 / squareSize;
        var startk = Math.floor(-squareSize / 2);
        for (var k = startk; k < startk + squareSize; k++) {
            var startl = Math.floor(-squareSize / 2);
            for (var l = startl; l < startl + squareSize; l++) {
                var newCube = new THREE.Mesh(this.geometry, this.material);
                if (addedCubesCount < cubesCount) {
                    newCube.position.x = (l + 1) * xdelta;
                    newCube.position.y = (k + 1) * ydelta;
                    newCube.position.z = 0;

                    newCube.rotation.x = Math.random();
                    newCube.rotation.y = Math.random();
                    newCube.rotation.z = Math.random();
                    addedCubesCount++;

                    this.cubes.push(newCube);
                    this.scene.add(newCube);
                } else {
                    break;
                }
            }

        }
    }

    /*
    Postara se o vykresleni s animaci a aktualizuje statistiky. 
    */
    renderWithAnimation() {

        requestAnimationFrame(() => this.renderWithAnimation());
        if (this.controls.animation) {
            for (var i = 0; i < this.cubes.length; i++) {
                switch (i % 3) {
                    case 0:
                        this.cubes[i].rotation.x += Math.PI / 180;
                        break;
                    case 1:
                        this.cubes[i].rotation.y += Math.PI / 180;
                        break;
                    case 2:
                        this.cubes[i].rotation.z += Math.PI / 180;
                        break;
                }
            }
        }
        this.stats.update();
        this.statsMs.update();
        this.renderer.render(this.scene, this.camera);
    }

    /*
    Prida ovladani GUI. 
    */
    addControls(): void {
        var gui = new dat.GUI();

        var controller = gui.add(this.controls, 'cubesCount', 1, 5000).step(1);
        controller.setName('Počet krychlí');

        var controllerPre = gui.add(this.controls, 'cubesCount', [1, 50, 500, 1000, 2000, 5000]);
        controllerPre.setName('Počet krychlí');

        var controllerAnimation = gui.add(this.controls, 'animation');
        controllerAnimation.setName('Animace');

        var controllerTest = gui.add(this.controls, 'test');
        controllerTest.setName('Test');
        var controllerTestAnimated = gui.add(this.controls, 'testAnimated');
        controllerTestAnimated.setName('Test - animace');

        controller.onFinishChange((value) => this.addCubes(value));
        controllerPre.onFinishChange((value) => this.addCubes(value));
    }

    /*
    Prida kameru. 
    */
    addCamera(): void {
        this.camera = new THREE.PerspectiveCamera(45, window.innerWidth / window.innerHeight, 0.01, 200);
        this.camera.position.set(0, 0, 20);
    }

    /*
    Definuje krychli (geometrie, material) 
    */
    defineCube(): void {
        this.material = new THREE.MeshLambertMaterial({ color: 0x5bc2ce });
        this.geometry = new THREE.BoxGeometry(1, 1, 1, 1, 1, 1);
    }

    /*
    Prida svetla do sceny. 
    */
    addLights(): void {
        var light = new THREE.DirectionalLight(0xffffff, 1);
        light.position.set(0.5, 0.5, 1);
        this.scene.add(light);
    }

    /*
    Zmena poctu krychli pri spusteni testu. 
    */
    changeCubesCount(animated): void {
        var self = this;
        if (!animated) {
            self.controls.animation = false;
        } else {
            self.controls.animation = true;
        }

        for (var i = 0; i <= self.testCounts.length; i++)
            setTimeout((j) => {
                if (j > 0) {
                    var medianFps = self.stats.getFpsMedian();
                    var medianMs = self.stats.getMsMedian();
                    console.log(self.testCounts[j - 1] + ": " + medianFps.toString() + ", " + medianMs.toString());
                    if (j === self.testCounts.length) {
                        alert("Test dokončen.");
                        self.addCubes(self.testCounts[0]);
                        self.controls.animation = false;
                    }
                }
                if (j < self.testCounts.length) {
                    self.addCubes(self.testCounts[j]);
                }
            }, 10000 * i, i);
    }

    /*
    Vypsani FPS a MS dat do konzole. 
    */
    writeFps(count): void {
        var self = this;
        setTimeout((j) => {
            var medianFps = self.stats.getFpsMedian();
            var medianMs = self.stats.getMsMedian();
            console.log(j.toString() + ": " + medianFps.toString() + ", " + medianMs.toString());
        }, 17000, count);
    }
}

window.onload = () => {
    var app = new AppThreeFps();
    app.main();
}; 